@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import logging
 import urlparse
 
@@ -14,6 +14,11 @@ __metaclass__ = PoolMeta
 logger = logging.getLogger(__name__)
 section = 'ldap_authentication'
 
+# Old version of urlparse doesn't parse query for ldap
+# see http://bugs.python.org/issue9374
+if hasattr(urlparse, 'uses_query') and 'ldap' not in urlparse.uses_query:
+    urlparse.uses_query.append('ldap')
+
 
 def parse_ldap_url(uri):
     unquote = urlparse.unquote
@@ -39,14 +44,14 @@ def ldap_connection():
         scheme, port = 'ldap', 389
     conn = ldap.initialize('%s://%s:%s/' % (
             scheme, uri.hostname, uri.port or port))
-    if config.getboolean(section, 'active_directory', 'False'):
+    if config.getboolean(section, 'active_directory', default=False):
         conn.set_option(ldap.OPT_REFERRALS, 0)
     if 'tls' in uri.scheme:
         conn.start_tls_s()
 
-    bindname = extensions.get('bindname')
+    bindname, = extensions.get('bindname', [None])
     if not bindname:
-        bindname = extensions.get('!bindname')
+        bindname, = extensions.get('!bindname', [None])
     if bindname:
         # XXX find better way to get the password
         conn.simple_bind_s(bindname, config.get(section, 'bind_pass'))
@@ -78,7 +83,7 @@ class User:
             'onelevel': ldap.SCOPE_ONELEVEL,
             'subtree': ldap.SCOPE_SUBTREE,
             }.get(scope)
-        uid = config.get(section, 'uid', 'uid')
+        uid = config.get(section, 'uid', default='uid')
         if filter_:
             filter_ = '(&(%s=%s)%s)' % (uid, login, filter_)
         else:
@@ -102,10 +107,10 @@ class User:
                 if cls.ldap_search_user(login, con, attrs=[]):
                     find = True
                     break
-        except ldap.LDAPError, e:
-            logger.error('LDAPError: %s' % str(e))
+        except ldap.LDAPError:
+            logger.error('LDAPError when checking password', exc_info=True)
         if find:
-            cls.raise_user_error('set_passwd_ldap_user', (login.rec_name,))
+            cls.raise_user_error('set_passwd_ldap_user', (login,))
 
     @classmethod
     def create(cls, vlist):
@@ -134,7 +139,7 @@ class User:
                 con = ldap_connection()
                 if con:
                     user = cls(Transaction().user)
-                    uid = config.get(section, 'uid', 'uid')
+                    uid = config.get(section, 'uid', default='uid')
                     users = cls.ldap_search_user(user.login, con, attrs=[uid])
                     if users and len(users) == 1:
                         [(dn, attrs)] = users
@@ -144,8 +149,9 @@ class User:
                             del values['password']
                         else:
                             cls.raise_user_error('wrong_password')
-            except ldap.LDAPError, e:
-                logger.error('LDAPError: %s' % str(e))
+            except ldap.LDAPError:
+                logger.error('LDAPError when setting preferences',
+                    exc_info=True)
         super(User, cls).set_preferences(values, old_password=old_password)
 
     @classmethod
@@ -155,21 +161,23 @@ class User:
         try:
             con = ldap_connection()
             if con:
-                uid = config.get(section, 'uid', 'uid')
+                uid = config.get(section, 'uid', default='uid')
                 users = cls.ldap_search_user(login, con, attrs=[uid])
                 if users and len(users) == 1:
                     [(dn, attrs)] = users
                     if password and con.simple_bind_s(dn, password):
+                        # Use ldap uid so we always get the right case
+                        login = attrs.get(uid, [login])[0]
                         user_id, _ = cls._get_login(login)
                         if user_id:
                             LoginAttempt.remove(login)
                             return user_id
                         elif config.getboolean(section, 'create_user'):
                             user, = cls.create([{
-                                        'name': attrs.get(uid, [login])[0],
+                                        'name': login,
                                         'login': login,
                                         }])
                             return user.id
-        except ldap.LDAPError, e:
-            logger.error('LDAPError: %s' % str(e))
+        except ldap.LDAPError:
+            logger.error('LDAPError when login', exc_info=True)
         return super(User, cls).get_login(login, password)
